@@ -3,19 +3,20 @@
 #' A suite of bivariate plots with "px" on the horizontal axis and "pz" on the vertical axis.
 #'
 #' @param data PITCHf/x data to be visualized.
-#' @param geom plotting geometry. Current choices are: "point", "hex", "bin", "tile" and "subplot2d". 
+#' @param geom plotting geometry. Current choices are: "point", "hex", "bin" and "tile" 
 #' @param contour logical. Should contour lines be included?
 #' @param point.size Size of points (when geom="point")
 #' @param point.alpha plotting transparency parameter (when geom="point").
 #' @param color variable used to define coloring scheme.
 #' @param fill variable used to define subplot scheme (when geom="subplot2d").
 #' @param layer list of other ggplot2 (layered) modifications.
-#' @param model A function specifying a model which is used to predict upon a grid defined by \code{limitz}.
-#' It is recommended that one uses \link{gam} to plot response surface of a Generalized Additive Model (see examples). 
-#' It shouldn't be necessary to specify \code{data} within the \code{model} function since it will inherit from \code{strikeFX}. 
+#' @param model Either a \link{gamObject} or a call to fit a model via \link{gam} or \link{bam}.
+#' Note that the horizontal and vertical location of the pitch MUST be included as covariates named "px" and "pz", respectively.
+#' Relevant factor variables must also be included as covariates in order to produce facetted or differenced plot(s).
 #' If this option is used, the geometry must be either "hex", "tile" or "bin". If a non-valid geometry is used, the geometry will be forced to "tile".
-#' @param density1 List of length one that defines a density estimate. The name should correspond to a variable in \code{data}. The value should correspond to an (observed) value of that variable.
-#' @param density2 Similar to \code{density1}. If \code{density1 != density2}, the density estimates are automatically differenced.
+#' @param model.save logical. Save the fitted \code{model}? If TRUE, the relevant model object is \link{assign}ed to the global environment
+#' @param density1 List of length one. The name should correspond to a variable in \code{data}. The value should correspond to an (observed) value of that variable.
+#' @param density2 Similar to \code{density1}. If \code{density1 != density2}, the relevant estimates are automatically differenced.
 #' @param limitz limits for horizontal and vertical axes. 
 #' @param adjust logical. Should vertical locations be adjusted according to batter height?
 #' @param draw_zones logical. Should strikezones be included?
@@ -24,28 +25,31 @@
 #' @return Returns a ggplot2 object.
 #' @export
 #' @import mgcv
-#' @import plyr
+#' @import ggplot2
+#' @import hexbin
 #' @importFrom MASS kde2d
 #' @examples
 #' data(pitches)
-#' strikeFX(pitches, geom="tile", layer=facet_grid(.~stand))
+#' p <- strikeFX(pitches, geom="tile", layer=facet_grid(.~stand))
+#' p+theme(aspect.ratio=1)
 #' \dontrun{
 #' strikeFX(pitches, geom="hex", density1=list(des="Called Strike"), density2=list(des="Ball"), 
 #'          draw_zones=FALSE, contour=TRUE, layer=facet_grid(.~stand))
 #' noswing <- subset(pitches, des %in% c("Ball", "Called Strike"))
 #' noswing$strike <- as.numeric(noswing$des %in% "Called Strike")
-#' strikeFX(noswing, model=gam(strike ~ s(px)+s(pz), family = binomial(link='logit')), 
-#'          layer=facet_grid(.~stand))
+#' m1 <- mgcv::bam(strike ~ s(px, pz, by=factor(stand)) + 
+#'                factor(stand), data=noswing, family = binomial(link='logit'))
+#' strikeFX(noswing, model=m1, layer=facet_grid(.~stand))
 #' #If sample size is an issue, try increasing the binwidths
-#' strikeFX(noswing, geom="bin", model=gam(strike ~ s(px)+s(pz), family = binomial(link='logit')), 
-#'          layer=facet_grid(.~stand), binwidth=c(.5, .5))
-#' strikeFX(noswing, geom="bin", model=gam(strike ~ s(px)+s(pz), family = binomial(link='logit')), 
-#'          density1=list(top_inning="Y"), density2=list(top_inning="N"), layer=facet_grid(.~stand), 
-#'          binwidth=c(.5, .5))
+#' strikeFX(noswing, model=m1, layer=facet_grid(.~stand), binwidth=c(.5,.5))
+#' m2 <- mgcv::bam(strike ~ s(px, pz, by=factor(stand)) + s(px, pz, by=factor(top_inning)) + 
+#'            factor(stand) + factor(top_inning), data=noswing, family = binomial(link='logit'))
+#' strikeFX(noswing, geom="bin", model=m2, density1=list(top_inning="Y"), 
+#'          density2=list(top_inning="N"), layer=facet_grid(.~stand), binwidth=c(.5, .5))
 #' }
 #' 
 
-strikeFX <- function(data, geom = "point", contour=FALSE, point.size=3, point.alpha=1/3, color = "pitch_types", fill = "des", layer = list(), model, density1=list(), density2=list(), limitz=c(-2.5, 2.5, 0, 5), adjust=FALSE, draw_zones=TRUE, parent=FALSE, ...){ 
+strikeFX <- function(data, geom = "point", contour=FALSE, point.size=3, point.alpha=1/3, color = "pitch_types", fill = "des", layer = list(), model, model.save=TRUE, density1=list(), density2=list(), limitz=c(-2.5, 2.5, 0, 5), adjust=FALSE, draw_zones=TRUE, parent=FALSE, ...){ 
   px=pz_adj=..density..=top=bottom=right=left=x=y=z=NULL #ugly hack to comply with R CMD check
   if (any(!geom %in% c("point", "bin", "hex", "tile", "subplot2d"))) warning("Current functionality is designed to support the following geometries: 'point', 'bin', 'hex', 'tile', 'subplot2d'.")
   if ("pitch_type" %in% names(data)) { #Add descriptions as pitch_types
@@ -95,51 +99,118 @@ strikeFX <- function(data, geom = "point", contour=FALSE, point.size=3, point.al
     white_zone <- NULL
     black_zone <- NULL
   }
+  #As of version 0.7, we assume the model is begin fit via gam/bam
+  #Most of this is modeled after ?mgcv::vis.gam (view is 'px' and 'pz', non-facets are conditioned upon)
+  #Does it make sense to look at differenced grids given a pre-fitted model????
   if (!missing(model)) {
-    FX1 <- subsetFX(FX, density1)
-    form <- as.list(substitute(model))
-    x.grid <- seq(limitz[1], limitz[2], 0.05) #how to add option for grid granularity?
-    y.grid <- seq(limitz[3], limitz[4], 0.05) #how to add option for grid granularity?
+    #figure out whether model argument is a fitted model or if it has yet to be fitted
+    classy <- tryCatch(class(model), error = function(e) NULL)
+    isfit <- !is.null(classy)
+    if (!isfit) { #fit the model
+      form <- as.list(substitute(model))
+      fit <- fitModel(FX, form, model.save)
+    } else {
+      fit <- model
+    }
+    #create a 2D grid (eventually passed to the ?predict.gam method)
+    x.grid <- seq(limitz[1], limitz[2], 0.05) #add option for grid granularity?
+    y.grid <- seq(limitz[3], limitz[4], 0.05) 
     grid <- expand.grid(px=x.grid, pz=y.grid)
-    if (identical(density1, density2)) { #densities are not differenced
-      if (!is.null(facets)) {
-        stuff <- dlply(FX1, facets, function(x) { fitModel(x, form, grid) } )
-        densities <- ldply(stuff)
-      } else {
-        densities <- fitModel(FX1, form, grid)
+    #extract covariates included in gam/bam (see ?gamObject)
+    vars <- names(fit$var.summary)
+    #Summaries for factors have a single value. Other types have 3 values and we want to condition on the 2nd value (see var.summary of ?gamObject)
+    var_summary <- lapply(fit$var.summary, function(x) { if (length(x) == 1) x else x[2] })
+    factors <- vars[sapply(fit$var.summary, length) == 1]
+    ind <- vars %in% c("px", "pz")
+    extras <- vars[!ind] #we need to either condition on or facet by these variables
+    if (!all(c("px", "pz") %in% vars)) warning("The horizontal location and vertical locations of each pitch should included as covariates in your model as 'px' and 'pz'.")
+    #old attempt to extract covariates relevant for facetting
+    #byes <- sapply(model$smooth, function(x) x$by)
+    #tmp <- gsub(".*\\(", "", byes)            #remove anything before "("
+    #faucets <- unique(gsub("\\)", "", tmp))   #compare this to facets
+
+    #check if the facetting scheme makes sense for the model. if not, give warning and throw out facet call
+    if (!is.null(facets)){ #facets exist
+      if (length(factors) == 0){ #facets not possible!
+        warning("Facet layer will be ignored since no factor variables were put into the model.")
+        facets <- NULL
+        id <- sapply(lapply(layer, class), function(x) which("facet" %in% x)) 
+        layer[id > 0] <- NULL #get rid of the facet layer
+      } else { #factors variables do exist
+        ind <- facets %in% factors 
+        idx <- which(!ind) #which facet variables are not included?
+        if (length(idx) > 0) {
+          warning(paste0("Facet layer will be ignored since the following variables included in the model as factors: ", facets[idx]))
+          facets <- NULL
+          id <- sapply(lapply(layer, class), function(x) which("facet" %in% x)) 
+          layer[id > 0] <- NULL #get rid of the facet layer
+        }
       }
-      p <- plotDensity(densities, boundaries, contour, geom, ...)
-      p <- p + white_zone
-      return(p+labelz+xrange+yrange+layers)
-    } else {  #densities are differenced
-      FX2 <- subsetFX(FX, density2)
-      if (!is.null(facets)) {
-        stuff <- dlply(FX1, facets, function(x) { fitModel(x, form, grid) } )
-        stuff2 <- dlply(FX2, facets, function(x) { fitModel(x, form, grid) } )
-        densities <- ldply(stuff)
-        densities2 <- ldply(stuff2)
-      } else {
-        densities <- fitModel(FX1, form, grid)
-        densities2 <- fitModel(FX2, form, grid)
+    }
+    #'fixed' variables can have multiple 'conditioned' values (px, pz are also here since they are part of the 'view')
+    fixed <- c(facets, c("px", "pz"))
+    if (length(density1) > 0) fixed <- c(fixed, names(density1))
+    if (length(density2) > 0) fixed <- c(fixed, names(density2))
+    #'given' variables have one 'conditioned' value (should be the mode for factors and closest obs. to the median for numerics)
+    givens <- var_summary[!names(var_summary) %in% fixed]
+    for (i in seq_along(givens)) {
+      grid[names(givens)[i]] <- givens[[i]]
+    }
+    #now fit the model for each unique value necessary for facetting
+    if (!is.null(facets)) {
+      sheet <- NULL
+      for (i in facets) {
+        vals <- unique(FX[,i])
+        grid[i] <- NA
+        for (j in vals) {
+          grid[i] <- j
+          if (identical(density1, density2)) { #probabilities are not differenced
+            grid$z <- predict(fit, grid, type="response") #add an option to look at se!?! see ?predict.gam
+          } else { #probabilities are differenced
+            grid1 <- grid
+            grid1[names(density1)] <- density1[[1]]
+            z1 <- predict(fit, grid1, type="response")
+            grid2 <- grid
+            grid2[names(density2)] <- density2[[1]]
+            z2 <- predict(fit, grid2, type="response")
+            grid$z <- z1 - z2
+          }
+          sheet <- rbind(sheet, grid)
+        }
       }
-      diff <- densities[-grep("z", names(densities))]
-      diff$z <- densities$z - densities2$z
-      t2 <- plotDensity(diff, boundaries, contour, geom, ...)
-      t2 <- t2 + black_zone
-      return(t2+scale_fill_gradient2(midpoint=0)+labelz+xrange+yrange+layers)
+    } else { #no facets
+      if (identical(density1, density2)) { #probabilities are not differenced
+        grid$z <- predict(fit, grid, type="response") #add an option to look at se!?! see ?predict.gam
+      } else { #probabilities are differenced
+        grid1 <- grid
+        grid1[names(density1)] <- density1[[1]]
+        z1 <- predict(fit, grid1, type="response")
+        grid2 <- grid
+        grid2[names(density2)] <- density2[[1]]
+        z2 <- predict(fit, grid2, type="response")
+        grid$z <- z1 - z2
+      }
+      grid$z <- predict(fit, grid, type="response")
+      sheet <- grid
     }
+    
+    p <- plotDensity(sheet, boundaries, contour, geom, ...)
+    p <- p + white_zone
+    if (any(sheet$z < 0)) col_scale <- scale_fill_gradient2(midpoint=0) else col_scale <- NULL
+    return(p+labelz+xrange+yrange+layers+col_scale)
   }
-  if (geom %in% "subplot2d") { #special handling for subplotting
-    if (!require(ggsubplot)) {
-      message("The 'subplot2d' geom requires library(subplot2d)!")
-      return()
-    }
-    require(ggsubplot)
-    return(ggplot(data=FX)+labelz+xrange+yrange+
-             geom_subplot2d(aes(x=px, y=pz_adj, 
-                    subplot = geom_bar(aes_string(x=fill, fill = fill))), ...)+
-             black_zone+layers)
-  }
+# DAMN YOU CRAN!
+#   if (geom %in% "subplot2d") { #special handling for subplotting
+#     if (!require(ggsubplot)) {
+#       message("The 'subplot2d' geom requires library(ggsubplot)!")
+#       return(NULL)
+#     }
+#     require(ggsubplot)
+#     return(ggplot(data=FX)+labelz+xrange+yrange+
+#              geom_subplot2d(aes(x=px, y=pz_adj, 
+#                     subplot = geom_bar(aes_string(x=fill, fill = fill))), ...)+
+#              black_zone+layers)
+#   }
   if (geom %in% c("bin", "hex", "tile")) { #special handling for (2D) density geometries
     if (identical(density1, density2)) { #densities are not differenced
       FX1 <- subsetFX(FX, density1)
@@ -176,19 +247,31 @@ strikeFX <- function(data, geom = "point", contour=FALSE, point.size=3, point.al
 }
 
 #fit a model and return the predicted values for a (possibly differenced) 2D grid
-fitModel <- function(dat, expr, grid) {
+fitModel <- function(dat, expr, model.save=TRUE) {
   fun <- as.character(expr[[1]])
   expr[[1]] <- NULL
   expr$data <- dat #overwrite data argument
   fit <- do.call(fun, expr)
-  pred.grid <- predict(fit, grid, type="response")
-  grid.df <- data.frame(x=grid[,1], y=grid[,2], z=pred.grid)
-  grid.df
+  if (model.save){
+    suffix <- gsub(":| ", "-", format(Sys.time(), "%m %d %X")) #month-day-hour-minute-second
+    filename <- paste0("model-", suffix, ".rds")
+    saveRDS(fit, file=filename)
+    message(paste0("Saved model fit as: ", filename))
+    Sys.sleep(1) #ensure we get unique filenames
+  }
+  fit
+  #meanFun(fit, grid)
 }
 
+# #use fitted model to find mean response over a 2D grid
+# meanFun <- function(fit, grid, se=FALSE) {
+#   pred.grid <- predict(fit, grid, type="response", se)
+#   data.frame(x=grid[,1], y=grid[,2], z=pred.grid)
+# }
+
 #Plot a 2D density with pre-computed heights
 plotDensity <- function(dens, bounds, contour, geom, ...){
-  px=pz_adj=..density..=top=bottom=right=left=x=y=z=NULL #ugly hack to comply with R CMD check
+  px=pz=pz_adj=..density..=top=bottom=right=left=x=y=z=NULL #ugly hack to comply with R CMD check
   common <- intersect(names(dens), names(bounds[[2]]))
   if (length(common) == 0) { #Artifically create a variable (stand) if none exists (required for joining and thus drawing strikezones)
     nhalf <- dim(dens)[1]/2
@@ -201,11 +284,11 @@ plotDensity <- function(dens, bounds, contour, geom, ...){
   dens.df <- join(dens, bounds[[2]], type="inner") #defaults to join "by" all common variables
   p <- ggplot(data=dens.df)
   if (geom %in% "hex") {
-    p <- p + stat_summary_hex(aes(x=x,y=y,z=z), ...)
+    p <- p + stat_summary_hex(aes(x=px, y=pz, z=z), ...)
   } else {
-    p <- p + stat_summary2d(aes(x=x,y=y,z=z), ...) 
+    p <- p + stat_summary2d(aes(x=px, y=pz, z=z), ...) 
   }
-  if (contour) p <- p + stat_contour(aes(x=x,y=y,z=z)) #passing binwidth here throws error
+  if (contour) p <- p + stat_contour(aes(x=px, y=pz, z=z)) #passing binwidth here throws error
   return(p)
 }
 
@@ -242,7 +325,7 @@ diffDensity <- function(data, density1, density2, limz){
   data1 <- subsetFX(data, density1)
   data2 <- subsetFX(data, density2)
   est1 <- kde2d(data1[,"px"], data1[,"pz_adj"], n=100, lims=limz)
-  grid <- expand.grid(x = est1$x, y = est1$y)
+  grid <- expand.grid(px = est1$x, pz = est1$y)
   est2 <- kde2d(data2[,"px"], data2[,"pz_adj"], n=100, lims=limz)
   grid["z"] <- as.vector(est1$z) - as.vector(est2$z)
   return(grid)
